<?php
/**
* @version $Id: gpx.php 526 2018-02-21 22:00:30Z eska $
* @package phpmygpx
* @copyright Copyright (C) 2009-2018 Sebastian Klemm.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

define( '_VALID_OSM', TRUE );
define( '_PATH', './' );
define( 'GPX_UPLOAD_DIR', './upload/');
define( 'SQL_BIG_SELECTS', 0);
$DEBUG = FALSE;
if($DEBUG) error_reporting(E_ALL);

session_start();
ob_start();

include("./check_db.php");
include("./libraries/mysql.classes.php");
include("./libraries/classes.php");
include("./libraries/html.classes.php");
include("./libraries/GeoCalc.class.php");
include("./gpx.html.php");

settimezone();
setlocale(LC_TIME, $cfg['config_locale']);
include("./languages/".get_lang($cfg['config_language']).".php");
include("./head.html.php");

if($cfg['show_exec_time'])
    $startTime = microtime_float();

if($DEBUG) {
    foreach($_POST as $akey => $val)
        out("<b>$akey</b> = $val", "OUT_DEBUG");
}

$sort 			= getUrlParam('HTTP_GET', 'INT', 's');
$order 			= getUrlParam('HTTP_GET', 'INT', 'o');
$limit 			= getUrlParam('HTTP_GET', 'INT', 'l');
$p 				= getUrlParam('HTTP_GET', 'INT', 'p');
$id 			= getUrlParam('HTTP_GET', 'INT', 'id');
$task 			= getUrlParam('HTTP_GET', 'STRING', 'task');
$option 		= getUrlParam('HTTP_GET', 'STRING', 'option');
$date_from	 	= getUrlParam('HTTP_GET', 'STRING', 'date_from');
$date_to 		= getUrlParam('HTTP_GET', 'STRING', 'date_to');
$f_lat			= getUrlParam('HTTP_GET', 'FLOAT', 'lat');
$f_lon			= getUrlParam('HTTP_GET', 'FLOAT', 'lon');
$f_lat_range 	= getUrlParam('HTTP_GET', 'FLOAT', 'lat_range');
$f_lon_range 	= getUrlParam('HTTP_GET', 'FLOAT', 'lon_range');
$description 	= getUrlParam('HTTP_GET', 'STRING', 'description');
$submit			= getUrlParam('HTTP_POST', 'STRING', 'submit_state');
$confirm		= getUrlParam('HTTP_POST', 'STRING', 'confirm');

// convert lat/lon from float value to database integer format
if($option == "search") {
	$option = "filter";
	$f_lat *= 1000000; 
	$f_lon *= 1000000; 
	$f_lat_range *= 1000000; 
	$f_lon_range *= 1000000; 
}

// handling of sort preference
$sort = handlePreference($sort, 'gpx_sort');

// handling of order preference
$order = handlePreference($order, 'gpx_order');

// handling of page number (for pagination)
$p = handlePreference($p, 'gpx_page');

// handling of sql result limit
if($limit) {
	// calc new page number caused by changed result limit
	$p = round($p * $_COOKIE['gpx_limit'] / $limit);
	setcookie('gpx_page', $p, time() + 3600 * $cfg['pref_cookie_lifetime']);
}
$limit = handlePreference($limit, 'gpx_limit');
// use default from config file if not set in cookie or URL GET parameter
if(!$limit)
	$limit = $cfg['result_table_limit'];

// connect to database
$database = new Mysql($cfg['db_host'], $cfg['db_name'], 
					$cfg['db_user'], $cfg['db_password']);

if(!$cfg['embedded_mode'] || !$cfg['public_host'] || check_password($cfg['admin_password'])) {
	HTML::heading(_APP_NAME, 2);
	HTML::main_menu();
}

switch ($task) {
	case 'view':
		viewGPX($p, $sort, $order, $limit);
		break;

	case 'details':
		details($id);
		break;

	case 'upload':
		uploadGPX();
		break;

	case 'batchimport':
		batchimportGPX();
		break;

	case 'import':
		importGPX();
		break;

	case 'edit':
		editGPX($id, $submit);
		break;

	case 'delete':
		deleteGPX($id, $submit, $confirm);
		break;

	case 'search':
		searchGPX();
		break;

	default:
		viewGPX(NULL, $sort, $order, $limit);
		break;
}


function viewGPX($page, $sort, $order, $limit) {
	global $DEBUG, $cfg, $database;
	global $option, $description, $date_from, $date_to, 
		$f_lat, $f_lon, $f_lat_range, $f_lon_range;
    HTML::heading(_MENU_GPX_VIEW, 3);
	$search_url = "";
	
	switch($sort) {
		case '1':
			$qs = 'gpx.id';
			break;
		case '2':
			$qs = 'gpx.size';
			break;
		case '4':
			$qs = 'gpx.timestamp';
			break;
		case '5':
			$qs = 'gpx.length';
			break;
		case '3':
		default:
			$qs = 'mints';
			$sort = 3;
			break;
	}
	
	if(!$order)
		$qo = 'ASC';
	else
		$qo = 'DESC';
	
	// prepare subquery conditions
	$subquery_conditions = "WHERE 1 ";
	if($option == "filter") {
		if(strtotime($date_from) || strtotime($date_to)) {
			if(!strtotime($date_from))
				$from = 0; // 1970-01-01
			else
				$from = strtotime($date_from);
			if(!strtotime($date_to))
				$to = 'UNIX_TIMESTAMP(NOW())';
			else
				$to = strtotime($date_to) + 24*3600; // add one day
			$subquery_conditions .= "AND UNIX_TIMESTAMP(`timestamp`) BETWEEN $from AND $to ";
		}
		if($f_lat)
			$subquery_conditions .= "AND `latitude` BETWEEN '".($f_lat - $f_lat_range)."' 
					AND '".($f_lat + $f_lat_range)."' ";
		if($f_lon)
			$subquery_conditions .= "AND `longitude` BETWEEN '".($f_lon - $f_lon_range)."' 
					AND '".($f_lon + $f_lon_range)."' ";
	}
	
	// set strict MySQL config, useful for optimizing SQL queries
	// could be removed as soon as bug FS#75 is closed
	if($DEBUG) {
		out('SQL_MAX_JOIN_SIZE=150000;', 'OUT_DEBUG');
		$result = $database->query("SET SQL_MAX_JOIN_SIZE=150000;");
		$result = $database->query("SET SQL_BIG_SELECTS=0;");
	}
	// workaround for bug FS#75, set SQL_BIG_SELECTS=1 on line 12 if necessary
	if(SQL_BIG_SELECTS) {
		$result = $database->query("SET SQL_BIG_SELECTS=1;");
	}
	
    $query = "SELECT SQL_CALC_FOUND_ROWS gpx.*, gpx.timestamp AS 'insert_ts', 
    				MIN(`pts`.`mintime`) AS 'mints', MAX(`pts`.`maxtime`) AS 'maxts' 
    			FROM `${cfg['db_table_prefix']}gpx_files` AS gpx INNER JOIN (
    				SELECT `gpx_id`, MIN(`timestamp`) mintime, MAX(`timestamp`) maxtime 
    					FROM `${cfg['db_table_prefix']}gpx_import` 
    					$subquery_conditions
    					GROUP BY `gpx_id` 
    				UNION 
    				SELECT `gpx_id`, MIN(`timestamp`) mintime, MAX(`timestamp`) maxtime 
    					FROM `${cfg['db_table_prefix']}waypoints` 
    					$subquery_conditions
    					GROUP BY `gpx_id` 
    				) AS pts 
    			ON gpx.id = pts.gpx_id ";
	if($option == "filter") {
		$query .= "WHERE 1 ";
		if($description)
			$query .= "AND gpx.description LIKE '%".$database->escape_string($description)."%' ";
		
		$search_url = "&option=filter&date_from=$date_from&date_to=$date_to&lat=$f_lat&lon=$f_lon&lat_range=$f_lat_range&lon_range=$f_lon_range&description=$description";
	}
	$query .= "GROUP BY pts.gpx_id ORDER BY $qs $qo LIMIT ". ($page*$limit) .", $limit ;";
    $result = $database->query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
    if(mysqli_num_rows($result)) {
		$num_found = $database->query("SELECT FOUND_ROWS();")->fetch_array()[0];
		if($option == "filter")
			HTML::message_r(_CMN_SEARCH_RESULTS, $num_found);
		
		HTML::viewPagination($page, ceil($num_found/$limit), 
			"gpx.php?task=view&s=$sort&o=$order".$search_url);
		HTML::viewResultLimitSelect("gpx.php?task=view&p=$page&s=$sort&o=$order".$search_url);
		
		HTML_gpx::viewGpxTableHeader("gpx.php?task=view&p=$page".$search_url, $sort, $order);
		while($row = mysqli_fetch_array($result, MYSQLI_ASSOC)) {
			HTML_gpx::viewGpxTableRow($row);
		}
		HTML_gpx::viewGpxTableFooter();
		
		HTML::viewPagination($page, ceil($num_found/$limit), 
			"gpx.php?task=view&s=$sort&o=$order".$search_url);
		if($option != "filter")
			HTML::message($num_found ._DB_GPX_AVAILABLE);
	}
	else {
		// if cookie has a invalid page number then reset it
		if($page > 0) {
			// redirect to 1st page (might be empty, if there are no gpx files)
			HTML::viewLocationRedirectJS('gpx.php?task=view&p=0');
		}
		else
			HTML::message(_TRC_NO_GPX_IN_DB);
	}
}

function details($id) {
	global $DEBUG, $cfg, $database;
    HTML::heading(_MENU_GPX_DETAILS, 3);
    #HTML::message(_TRC_DETAILS_OF_GPX . $id);
    $query = "SELECT * FROM `${cfg['db_table_prefix']}gpx_files` WHERE `id` = '$id' ;";
    $result = $database->query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
    if(mysqli_num_rows($result)) {
    	while($row = mysqli_fetch_array($result, MYSQLI_ASSOC)) {
			HTML_gpx::viewGpxDetails($row);
			$gpxStats = $row;
    	}
    	
    	// query stats for trackpoints
	    $query = "SELECT `gpx_id`, COUNT(*) AS 'trkpts', 
			MIN(`latitude`) AS 'minlat', MAX(`latitude`) AS 'maxlat', AVG(`latitude`) AS 'avglat', 
			MIN(`longitude`) AS 'minlon', MAX(`longitude`) AS 'maxlon', AVG(`longitude`) AS 'avglon',
			MIN(`altitude`) AS 'minalt', MAX(`altitude`) AS 'maxalt', AVG(`altitude`) AS 'avgalt', 
			MIN(`timestamp`) AS 'mints', MAX(`timestamp`) AS 'maxts',
			MIN(`sat`) AS 'minsat', MAX(`sat`) AS 'maxsat', AVG(`sat`) AS 'avgsat', 
			MIN(`hdop`) AS 'minhdop', MAX(`hdop`) AS 'maxhdop', AVG(`hdop`) AS 'avghdop', 
			MIN(`pdop`) AS 'minpdop', MAX(`pdop`) AS 'maxpdop', AVG(`pdop`) AS 'avgpdop'
			FROM `${cfg['db_table_prefix']}gpx_import` WHERE `gpx_id` = '$id' GROUP BY `gpx_id` ;";
	    $result = $database->query($query);
		if($DEBUG)	out($query, 'OUT_DEBUG');
	    if(mysqli_num_rows($result)) {
	    	while($row = mysqli_fetch_array($result, MYSQLI_ASSOC)) {
				$gpxStats = array_merge($gpxStats, $row);
	    	}
		}else
			$gpxStats['gpx_id'] = $id;
		
    	// query stats for waypoints
	    $query = "SELECT COUNT(*) AS 'wpts' FROM `${cfg['db_table_prefix']}waypoints` 
	    			WHERE `gpx_id` = '$id' ;";
	    $result = $database->query($query);
		if($DEBUG)	out($query, 'OUT_DEBUG');
	    if(mysqli_num_rows($result)) {
	    	while($row = mysqli_fetch_array($result, MYSQLI_ASSOC)) {
				$gpxStats['wpts'] = $row['wpts'];
	    	}
	    }
	    
    	// query stats for photos
	    $query = "SELECT COUNT(*) AS 'photos' FROM `${cfg['db_table_prefix']}pois` 
	    			WHERE `gpx_id` = '$id' ;";
	    $result = $database->query($query);
		if($DEBUG)	out($query, 'OUT_DEBUG');
	    if(mysqli_num_rows($result)) {
	    	while($row = mysqli_fetch_array($result, MYSQLI_ASSOC)) {
				$gpxStats['photos'] = $row['photos'];
	    	}
	    }
	    
	    HTML_gpx::viewGpxContentLinks($gpxStats);
	    
		// show details if trackpoints availabe
		if($gpxStats['trkpts']) {
		    HTML_gpx::viewGpxDetailsTable($gpxStats);
		
			// track distance calculation
			$stw = microtime_float();
			$distances = Trip::getDistancesArray($id);
			if($DEBUG)
				out("(dist calc took ". round(microtime_float() - $stw, 4) ." s)", 'OUT_DEBUG');
			HTML_gpx::viewGpxDistances($distances, $gpxStats['timezone']);
			
			// split elevation chart if breaks found
			if(sizeof($distances) > 2)
				HTML::message(_TRC_DETAILS_CHART_SPLIT);
			
			HTML::viewChartViewSelect();
			
			// draw elevation charts
			$chart_count = 1;
			foreach($distances as $key=>$val) {
				if(!$key)
					continue;
				if($cfg['chart_altitude_type'] == 'dist')
					$charttype = 'alt_dist';
				else
					$charttype = 'alt_time';
				
				echo '<a id="chart_link_'.$chart_count.'" href="./graph.php?type='.$charttype.
				'&alt=1&speed=1'.'&id='.$id.'&tz='.$gpxStats['timezone'].
				'&bts='.$distances[$key-1][2].'&ets='.$val[1].
				'&width='.$cfg['chart_big_width'].'&height='.$cfg['chart_big_height'].'">
				<img id="chart_'.$chart_count.'" src="./graph.php?type='.$charttype.
				'&alt=1&speed=1'.'&id='.$id.'&tz='.$gpxStats['timezone'].
				'&bts='.$distances[$key-1][2].'&ets='.$val[1].
				'&width='.$cfg['chart_width'].'&height='.$cfg['chart_height'].'"'.
				'width="'.$cfg['chart_width'].'" height="'.$cfg['chart_height'].'" '.
				'border="1" alt="GPX Altitude Profile" /></a>';
				
				$chart_count++;
			}
		}else
			HTML::message(_TRC_NO_TRKPTS_IN_DB);
    }else {
		HTML::message(_TRC_GPX_DOES_NOT_EXIST);
		return 1;
    }
}

function uploadGPX() {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_GPX_UPL, 3);
    if(!$cfg['public_host'] || check_password($cfg['admin_password'])) {
    	HTML_gpx::fileUploadForm('gpx.php?task=import', 
    		check_max_filesize($cfg['max_file_size']));
    }else {
    	HTML::message(_NOT_AUTH ." ". _DO_LOGIN);
    }
}

function batchimportGPX() {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_GPX_BATCH_IMPORT, 3);
    if(!$cfg['public_host'] || check_password($cfg['admin_password'])) {
	    echo "<script src='./libraries/import.js'></script>\n";
	    // set current upload dir and scan it for gpx files
		$dir = GPX_UPLOAD_DIR;
		$gpx_files = scan_dir_f($dir, 0, 'FILETYPE_FILE', 'GPX');
		
		HTML::message_r(_TRC_BATCH_IMPORTING_DIR, $dir);
		HTML::message(_TRC_CHOOSE_FILES_FOR_BATCH_IMPORTING);
		if($DEBUG)	print_r($gpx_files);
		
		HTML_gpx::viewBatchImportTableHeader();
		for($i=0; $i<sizeof($gpx_files); $i++) {
			HTML_gpx::viewBatchImportTableRow($i+1, $gpx_files[$i]);
		}
		HTML_gpx::viewBatchImportTableFooter($dir);
		HTML_gpx::viewImportProgress();
    }else {
    	HTML::message(_NOT_AUTH ." ". _DO_LOGIN);
    }
}

function importGPX() {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_GPX_IMPORT, 3);
    if(!$cfg['public_host'] || check_password($cfg['admin_password'])) {
	    echo "<script src='./libraries/import.js'></script>\n";
		if($DEBUG) {
		    out($_FILES['userfile']['name'] ." (file name)", 'OUT_DEBUG');
		    out($_FILES['userfile']['size'] ." Bytes (size)", 'OUT_DEBUG');
		    out($_FILES['userfile']['error'] ." (error number)", 'OUT_DEBUG');
		}
		// set current upload dir and move file there
		$dir = GPX_UPLOAD_DIR;
		$file = $dir. $_FILES['userfile']['name'];
		
	    if(!move_uploaded_file($_FILES['userfile']['tmp_name'], $file)) {
	    	switch ($_FILES['userfile']['error']) {
		        case UPLOAD_ERR_INI_SIZE:
		        case UPLOAD_ERR_FORM_SIZE:
		            $upl_error = _CMN_UPLOAD_ERR_SIZE;
		            break;
		        case UPLOAD_ERR_PARTIAL:
		            $upl_error = _CMN_UPLOAD_ERR_PARTIAL;
		            break;
		        case UPLOAD_ERR_NO_FILE:
		            $upl_error = _CMN_UPLOAD_ERR_NO_FILE;
		            break;
		        case UPLOAD_ERR_NO_TMP_DIR:
		            $upl_error = _CMN_UPLOAD_ERR_NO_TMP_DIR;
		            break;
		        case UPLOAD_ERR_CANT_WRITE:
		            $upl_error = _CMN_UPLOAD_ERR_CANT_WRITE;
		            break;
		        case UPLOAD_ERR_EXTENSION:
		        default:
		            $upl_error = 'Unknown upload error';
	    	}
	    	HTML::message(_TRC_UPL_ERROR . $upl_error);
	    }else {
	    	HTML::message(_TRC_UPL_SUCCESS);
			HTML_gpx::viewImportForm(1, basename($file), $dir, 
				strip_tags($_POST['description']), intval($_POST['tz']));
			HTML_gpx::viewImportProgress();
		}
    }else {
    	HTML::message(_NOT_AUTH ." ". _DO_LOGIN);
    }
}

function editGPX($id, $submit) {
	global $DEBUG, $cfg, $database;
    HTML::heading(_MENU_GPX_EDIT, 3);
    if(!$cfg['public_host'] || check_password($cfg['admin_password'])) {
	    if(!$submit) {
	        $query = "SELECT * FROM `${cfg['db_table_prefix']}gpx_files` 
	        	WHERE `id` = '$id' ;";
	        $result = $database->query($query);
	        $row = mysqli_fetch_array($result, MYSQLI_ASSOC);
	    	if(mysqli_num_rows($result))
	    		HTML_gpx::editForm('gpx.php?task=edit&id='.$id, $id, $row);
		    else
				HTML::message(_TRC_GPX_DOES_NOT_EXIST);
	    } else {
			// edit description of gpx file
			$description = $database->escape_string(strip_tags($_POST['description']));
	        $query = "UPDATE IGNORE `${cfg['db_table_prefix']}gpx_files` 
	        	SET `description` = '$description' WHERE `id` = '$id' ;";
	        $result = $database->query($query);
	        if($database->get_affected_rows()) {
	        	HTML::message(_TRC_GPX_EDITED);
	        	HTML::message("<a href='?task=details&id=${id}'>"._MENU_GPX_DETAILS."</a>");
	        }
	    }
    }else {
    	HTML::message(_NOT_AUTH ." ". _DO_LOGIN);
    }
}

function deleteGPX($id, $submit, $confirm) {
	global $DEBUG, $cfg, $database;
    HTML::heading(_MENU_GPX_DELETE, 3);
    if(!$cfg['public_host'] || check_password($cfg['admin_password'])) {
	    if(!$submit) {
	    	HTML_gpx::deleteForm('gpx.php?task=delete&id='.$id, $id);
	    } else {
			if($confirm == _CMN_YES) {
				// delete waypoints
		        $query = "DELETE IGNORE FROM `${cfg['db_table_prefix']}waypoints` 
		        	WHERE `gpx_id` = '$id' ;";
		        $result = $database->query($query);
		        $num_rows = $database->get_affected_rows();
		        HTML::message_r(_TRC_WPT_DELETED, $num_rows);
	
				// delete track points
		        $query = "DELETE IGNORE FROM `${cfg['db_table_prefix']}gpx_import` 
		        	WHERE `gpx_id` = '$id' ;";
		        $result = $database->query($query);
		        $num_rows = $database->get_affected_rows();
		        HTML::message_r(_TRC_TRKPT_DELETED, $num_rows);
	
				// delete gpx file
		        $query = "DELETE IGNORE FROM `${cfg['db_table_prefix']}gpx_files` 
		        	WHERE `id` = '$id' ;";
		        $result = $database->query($query);
		        $num_rows = $database->get_affected_rows();
		        HTML::message_r(_TRC_GPX_DELETED, $num_rows);
			} else {
				HTML::message(_TRC_NO_CONFIRM_DELETE);
			}    
	    }
    }else {
    	HTML::message(_NOT_AUTH ." ". _DO_LOGIN);
    }
}

function searchGPX() {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_GPX_SEARCH, 3);
    HTML_gpx::searchGpxForm('');
}


if($cfg['show_exec_time']) {
    $endTime = microtime_float();
    $exectime = round($endTime - $startTime, 4);
}
include("./foot.html.php");
?>
