<?php
/**
* @version $Id: html.classes.php 526 2018-02-21 22:00:30Z eska $
* @package phpmygpx
* @copyright Copyright (C) 2009-2018 Sebastian Klemm.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

class HTML {
	/**
	 * Dummy function for future usage.
	 */
	function html_head($str, $size) {
		$s = intval($size);
	}

	/**
	 * Dummy function for future usage.
	 */
	function html_foot($str, $size) {
		$s = intval($size);
	}

	/**
	 * Prints a heading in various sizes.
	 * @global Array $cfg Configuration.
	 * @param String $str Name of the heading.
	 * @param Int $size Size of heading (1...5).
	 * @return Void
	 */
	function heading($str, $size) {
		$s = intval($size);
		if($s > 0 && $s < 7)
			echo "<h$s>$str</h$s>\n";
	}

	/**
	 * Prints a paragraph containing a message.
	 * @param String $str Text of the paragraph.
	 * @return Void
	 */
    function message($str) {
    	HTML::message_r($str, NULL);
    }

	/**
	 * Prints a paragraph containing a message with replacing.
	 * Useful for translated strings containing variable contents.
	 * Special patterns like %1% ... %5% in the text will be replaced.
	 * @param String $str Text of the paragraph.
	 * @param Array $replace Up to 5 replacement strings.
	 * @return Void
	 */
    function message_r($str, $replace) {
    	if(!is_null($replace)) {
            $search = array("%1%","%2%","%3%","%4%","%5%");
    		$mod_str = str_replace($search, $replace, $str);
    		$str = $mod_str;
    	}
        echo "<p>$str</p>\n";
    }

	/**
	 * Prints a horizontal menu enclosed by <div> tags.
	 * @global Array $cfg An array holding all configuration variables.
	 * @return Void
	 */
    function main_menu() {
    	global $cfg;
		echo '<div id="menu">';
		if($cfg['gpx_features']) {
			echo '<span id="menu_category">'._MENU_GPX.':&nbsp;</span>';
			echo '<a href="gpx.php?task=view">'._MENU_VIEW.'</a> | ';
			if(!$cfg['public_host'] || check_password($cfg['admin_password']))
				echo '<a href="gpx.php?task=upload">'._MENU_UPL.'</a> | ';
			echo '<a href="gpx.php?task=search">'._MENU_SEARCH.'</a> | ';
			echo '<span id="menu_category">'._MENU_TRKPT.':&nbsp;</span>';
			echo '<a href="traces.php?task=view">'._MENU_VIEW.'</a> | ';
			echo '<a href="traces.php?task=search">'._MENU_SEARCH.'</a> | ';
			echo '<span id="menu_category">'._MENU_WPT.':&nbsp;</span>';
			echo '<a href="waypoints.php?task=view">'._MENU_VIEW.'</a> | ';
			echo '<a href="waypoints.php?task=search">'._MENU_SEARCH.'</a> | ';
			if(!$cfg['public_host'] || check_password($cfg['admin_password']))
				echo '<a href="waypoints.php?task=new">'._MENU_NEW.'</a> | ';
			echo '<br>';
		}
		if($cfg['photo_features']) {
			echo '<span id="menu_category">'._MENU_PHOTO.':&nbsp;</span>';
			echo '<a href="photos.php?task=view">'._MENU_VIEW.'</a> | ';
			if(!$cfg['public_host'] || check_password($cfg['admin_password']))
				echo '<a href="photos.php?task=upload">'._MENU_UPL.'</a> | ';
		}
		echo '<span id="menu_category">'._MENU_MISC.':&nbsp;</span>';
		echo '<a href="index.php">'._MENU_HOME.'</a> | ';
		echo '<a href="map.php">'._MENU_MAP.'</a> | ';
		echo '<a href="bookmark.php">'._MENU_BOOKMARK.'</a> | ';
		echo '<a href="rss_feed.php">'._MENU_RSS_FEED.'</a> | ';
		echo '<a href="database.php?task=stats">'._MENU_DB_STAT.'</a> | ';
		echo '<a href="about.php">'._MENU_ABOUT.'</a> | ';
		if($cfg['public_host']) {
			if(check_password($cfg['admin_password']))
				echo '[<a href="index.php?task=logout">'._MENU_LOGOUT.'</a>]';
			else
				echo '[<a href="index.php?task=login">'._MENU_LOGIN.'</a>]';
		}
		echo '</div>';
		echo '<br clear="all"> ';
    }

    function viewPagination($cur, $max, $url) {
		echo '<div class="pagination">';
		if($cur!=0) {
			echo ' <a href="'. $url .'&p=0">'. _CMN_FIRST ."</a> ";
			echo ' <a href="'. $url .'&p='. ($cur-1) .'">'. _CMN_PREV ."</a> ";
		}
		echo "[". _CMN_PAGE ." ". ($cur+1) ."/". $max ."]";
		if($cur < $max-1) {
			echo ' <a href="'. $url .'&p='. ($cur+1) .'">'. _CMN_NEXT ."</a> ";
			echo ' <a href="'. $url .'&p='. ($max-1) .'">'. _CMN_LAST ."</a> ";
		}
		echo "</div>\n";
    }

    function viewTableViewSelect($view, $url) {
		echo '<div class="viewSelect">';
		echo "["._CMN_VIEW.": <a href='${url}&view=simple'>"._CMN_VIEW_SIMPLE."</a>\n";
		echo " | <a href='${url}&view=detailed'>"._CMN_VIEW_DETAIL."</a>]\n";
		echo "</div>\n";
	}

    function viewChartViewSelect() {
		echo '<div class="viewSelect">';
		echo "["._CMN_VIEW.": <a href='javascript:changeChartType(\"time\")'>"._CHART_AXIS_TIME."</a>\n";
		echo " | <a href='javascript:changeChartType(\"dist\")'>"._CHART_AXIS_DIST."</a>]\n";
		echo "</div>\n";
	}

    function viewResultLimitSelect($url) {
    	global $limit;
		echo '<div class="viewSelect">';
		echo "<select size=1 name='l' onchange='changeResultLimit(this.options[this.selectedIndex].value, \"$url\");' >\n";
		echo "  <option value=0>"._CMN_SHOW.": $limit</option>\n";
		echo "  <option value=10>10</option>\n";
		echo "  <option value=15>15</option>\n";
		echo "  <option value=20>20</option>\n";
		echo "  <option value=25>25</option>\n";
		echo "  <option value=35>35</option>\n";
		echo "  <option value=50>50</option>\n";
		echo "</select>\n";
		echo "</div>\n";
	}

    function viewTimezoneSelect($selected) {
		echo '<div class="viewSelect">'. _CMN_TIMEZONE .': ';
		echo "<select size=1 name='tz' >\n";
		echo "  <option value=0>"._CMN_TIMEZONE.": $limit</option>\n";
		for($offset=-12; $offset<13; $offset++) {
			echo "  <option value='${offset}' ". ($offset==$selected?'selected="selected"':'')
				.">GMT". ($offset>0?'+':'') ."${offset}</option>\n";
		}
		echo "</select>\n";
		echo "</div>\n";
	}

	function viewTabColHead($url, $nr, $sort, $order, $name) {
		echo "<th><a title='". ($order ? _CMN_SORT_ASC : _CMN_SORT_DESC);
		echo "' href='$url&s=$nr&o=". (intval(!(bool)$order)) ."'>". $name;
		if($sort == $nr) {
			$arrow = array("&#9650;", "&#9660;");
			echo $arrow[intval($order)];
		}
		echo "</a></th>\n";
	}

	function getGpxLink($gpx) {
		if($gpx['gpx_id']) {
			$string = "<a href='gpx.php?task=details&id=${gpx['gpx_id']}' ";
			$string .= "title='${gpx['gpx_description']} ";
			$string .= "(${gpx['gpx_filename']})'>${gpx['gpx_id']}</a>";
		}
		else
			$string = " - ";
		return $string;
	}

	/**
	 * Prints a HTML form for searching using a geocoder.
	 * @return Void
	 */
	function viewGeocoderSearchForm() {
		echo "<form action='#' onSubmit='return submitSearch();' class='filter'>";
		echo "<input type='text' name='search_str' value='' 
			placeholder='"._MAP_SEARCH_ADDRESS_POI."' size=32 />";
		echo "<input type='button' name='search_btn' value='"._MENU_SEARCH."' onClick='doBasicSearchClick();' />\n";
		echo "<div id='divBasicSearchResults'></div>\n";
		echo "</form>\n";
	}

	/**
	 * Prints some embedded Javascript to redirect to another page.
	 * @param String $url The URL of the redirection target.
	 * @param Int $timeout Timeout in milliseconds until redirection.
	 * @return Void
	 */
	function viewLocationRedirectJS($url, $timeout=0) {
		echo "<script type='text/javascript'>\n";
		echo "window.setTimeout(\"window.location.href='${url}'\", ${timeout});\n";
		echo "</script>\n";
	}
}
?>
