<?php
/**
* @version $Id: mysql.classes.php 523 2018-02-11 21:26:09Z eska $
* @package phpmygpx
* @copyright Copyright (C) 2016-2018 Sebastian Klemm.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

class Mysql {
	private $die_on_error	= null;
	private $link			= null;
	private $result			= null;
	public $affected_rows	= null;
	public $errno			= null;
	public $error			= null;
	public $insert_id		= null;
	public $num_rows		= null;
	
	public function __construct($host, $db, $user, $pw, $die=TRUE) {
		$this->die_on_error = $die;
		$this->link = $this->connect($host, $db, $user, $pw);
	}
	
	public function __destruct() {
	}
	
	private function die_on_error($message='') {
		if($this->link) {
			$this->errno = mysqli_errno($this->link);
			$this->error = mysqli_error($this->link);
		}
		// Exit the script if chosen in config
		if($this->die_on_error)
			die ($this->errno.': '.$this->error.' '.$message);
	}
	
	private function connect($host, $db, $user, $pw) {
		$link = @mysqli_connect($host, $user, $pw);
		// Connection failed
		if(!$link) {
			$this->errno = mysqli_connect_errno();
			$this->error = mysqli_connect_error();
			$this->die_on_error();
		}
		// Connection established
		else {
			// Select database if parameter given on construct
			if($db) {
				mysqli_select_db($link, $db)
					or $this->die_on_error();
			}
			
			#echo 'Initial character set: ' .  mysqli_client_encoding($link) . "\n";
			mysqli_set_charset($link, 'utf8')
				or $this->die_on_error('Error: Unable to set the MySQL client character set.');
			#echo 'Current character set: ' .  mysqli_client_encoding($link) . "\n";
		}
		return $link;
	}
	
	public function select_db($name) {
		$ok = mysqli_select_db($this->link, $name)
			or $this->die_on_error();
		return $ok;
	}
	
	public function escape_string($string) {
		if(get_magic_quotes_gpc())
			$string = stripslashes($string);
		return mysqli_real_escape_string($this->link, $string);
	}
	
	public function query($query) {
		$result = mysqli_query($this->link, $query) 
			or $this->die_on_error();
		
		if($result !== TRUE) {
			$this->result = $result;
			$this->num_rows = mysqli_num_rows($result);
		}
		$this->affected_rows = mysqli_affected_rows($this->link);
		$this->insert_id = mysqli_insert_id($this->link);
		$this->errno = mysqli_errno($this->link);
		$this->error = mysqli_error($this->link);
		return $result;
	}
	
	public function get_link() {
		return $this->link;
	}
	
	public function get_affected_rows() {
		return $this->affected_rows;
	}
	
	public function get_error() {
		return $this->error;
	}
	
	public function get_insert_id() {
		return $this->insert_id;
	}
	
	public function get_num_rows() {
		return $this->num_rows;
	}
	
	public function close() {
		if($this->result)
			mysqli_free_result($this->result);
		mysqli_close($this->link);
	}
}

?>
