<?php
/**
* @version $Id: pois.php 527 2018-02-23 19:40:15Z eska $
* @package phpmygpx
* @copyright Copyright (C) 2009-2018 Sebastian Klemm.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

define( '_VALID_OSM', TRUE );
define( '_PATH', './' );
$DEBUG = FALSE;
if($DEBUG)	error_reporting(E_ALL);
else		error_reporting(E_ERROR);

session_start();

include("./config.inc.php");
include("./libraries/functions.inc.php");
include("./libraries/mysql.classes.php");
include("./libraries/html.classes.php");

setlocale (LC_TIME, $cfg['config_locale']);
include("./languages/".get_lang($cfg['config_language']).".php");

if($cfg['show_exec_time'])
    $startTime = microtime_float();

if($DEBUG) {
    foreach($_POST as $akey => $val)
        out("<b>$akey</b> = $val", "OUT_DEBUG");
}

$task 		= getUrlParam('HTTP_GET', 'STRING', 'task');
$f_minlat 	= getUrlParam('HTTP_GET', 'FLOAT', 'b') * 1000000;
$f_maxlat 	= getUrlParam('HTTP_GET', 'FLOAT', 't') * 1000000;
$f_minlon 	= getUrlParam('HTTP_GET', 'FLOAT', 'l') * 1000000;
$f_maxlon 	= getUrlParam('HTTP_GET', 'FLOAT', 'r') * 1000000;

// connect to database
$database = new Mysql($cfg['db_host'], $cfg['db_name'], 
					$cfg['db_user'], $cfg['db_password']);


switch ($task) {
	case 'getWpt':
		getWaypoints();
		break;

	case 'getPhotos':
		getPhotos($f_minlat, $f_maxlat, $f_minlon, $f_maxlon);
		break;

	default:
        getData();
		break;
}


function getWaypoints() {
	global $DEBUG, $cfg, $database;
	$limit = intval($cfg['result_datalayer_limit']);
    $query = "SELECT SQL_CALC_FOUND_ROWS wpt.* , `gpx`.`timezone`, 
				`gpx`.`name` AS 'gpx_filename', `gpx`.`description` AS `gpx_description` 
				FROM `${cfg['db_table_prefix']}waypoints` AS `wpt` 
				LEFT JOIN `${cfg['db_table_prefix']}gpx_files` AS `gpx` 
				ON `gpx_id` = `gpx`.`id` ";
    /*
	if($option == "filter") {
		$query .= "WHERE `name` LIKE '%$description%' ";
	}
	*/
	$query .= "ORDER BY `wpt`.`id` ASC LIMIT 0, $limit ;";
    $result = $database->query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
    if(mysqli_num_rows($result)) {
		#$num_found = $database->query("SELECT FOUND_ROWS();")->fetch_array()[0];
		
		// write table header for datalayer
		echo "lat\tlon\ttitle\tdescription\ticon\ticonSize\ticonOffset\n";
		
		// write table row for each waypoint
		while($row = mysqli_fetch_array($result, MYSQLI_ASSOC)) {
			$description = _CMN_ALT .": ${row['altitude']} m<br />";
			$description .= _CMN_LAT .": ".($row['latitude']/1000000)."<br />";
			$description .= _CMN_LON .": ".($row['longitude']/1000000)."<br />";
			$description .= "${row['timestamp']}<br />";
			$description .= _CMN_GPX_ID .": ".HTML::getGpxLink($row)."<br />";;
			$description .= str_replace("\n", "<br>", $row['cmt']). "<br />";
			$description .= str_replace("\n", "<br>", $row['desc']) ."<br />";
			
			# embedd audio player if soundmap enabled and audio file available
			if($cfg['soundmap_features'] && $row['audio_url']) {
				$description .= "<audio id='soundmap_audio' controls preload='metadata'><source src='".$row['audio_url']."' type='audio/mp3' />"._CMN_HTML5_AUDIO_NOT_SUPPORTED."</audio><br />";
			}
			# show icon for editing waypoint
			if(!$cfg['public_host'] || check_password($cfg['admin_password'])) {
				$description .= "<a href='waypoints.php?task=edit&id=${row['id']}'><img src='images/b_edit.png' /></a>";
			}
			
			// write table row for datalayer
			echo ($row['latitude']/1000000)."\t".($row['longitude']/1000000).
				"\t${row['name']} \t${description}\t${row['icon_file']}".
				"\t${row['icon_size']}\t${row['icon_offset']}\n";
		}
	}
}

function getPhotos($b, $t, $l, $r) {
	global $DEBUG, $cfg, $database;
	$limit = intval($cfg['result_datalayer_limit']);
    $query = "SELECT SQL_CALC_FOUND_ROWS * FROM `${cfg['db_table_prefix']}pois` 
    		WHERE `latitude` BETWEEN $b AND $t AND `longitude` BETWEEN $l AND $r ";
    /*
	if($option == "filter") {
		$query .= "WHERE `name` LIKE '%$description%' ";
	}
	*/
	$query .= "ORDER BY `id` ASC LIMIT 0, $limit ;";
    $result = $database->query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
    if(mysqli_num_rows($result)) {
		#$num_found = $database->query("SELECT FOUND_ROWS();")->fetch_array()[0];
		
		// write photo array for datalayer
		echo "[";
		$i = 1;
		while($row = mysqli_fetch_array($result, MYSQLI_ASSOC)) {
			if($i>1)
				echo ",";
			
			echo "{\"width\":\"0\",\"time\":\"${row['timestamp']}\",\"lat\":\"".lat2y($row['latitude']/1000000)."\",\"photoid\":\"${row['id']}\",\"lon\":\"".lon2x($row['longitude']/1000000)."\",\"height\":\"0\",\"viewdir\":\"${row['image_dir']}\"}";
			
			$i++;
		}
		echo "]";
	}
}


if($cfg['show_exec_time']) {
    $endTime = microtime_float();
    $exectime = round($endTime - $startTime, 4);
}


function lat2y($lat) {
	return (20037508.34 / 180) * (180/M_PI * log(tan(M_PI/4 + $lat*(M_PI/180)/2)));
}
function lon2x($lon) {
	return (20037508.34 / 180) * $lon;
}
?>
