<?php
/**
* @version $Id: traces.php 526 2018-02-21 22:00:30Z eska $
* @package phpmygpx
* @copyright Copyright (C) 2009-2018 Sebastian Klemm.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

define( '_VALID_OSM', TRUE );
define( '_PATH', './' );
define( 'GPX_UPLOAD_DIR', './upload/');
define( 'SQL_BIG_SELECTS', 0);
$DEBUG = FALSE;
if($DEBUG) error_reporting(E_ALL);

session_start();
ob_start();

include("./check_db.php");
include("./libraries/mysql.classes.php");
include("./libraries/classes.php");
include("./libraries/html.classes.php");
include("./libraries/GeoCalc.class.php");
include("./traces.html.php");

settimezone();
setlocale(LC_TIME, $cfg['config_locale']);
include("./languages/".get_lang($cfg['config_language']).".php");
include("./head.html.php");

if($cfg['show_exec_time'])
    $startTime = microtime_float();

if($DEBUG) {
    foreach($_POST as $akey => $val)
        out("<b>$akey</b> = $val", "OUT_DEBUG");
}

$sort 			= getUrlParam('HTTP_GET', 'INT', 's');
$order 			= getUrlParam('HTTP_GET', 'INT', 'o');
$limit 			= getUrlParam('HTTP_GET', 'INT', 'l');
$p 				= getUrlParam('HTTP_GET', 'INT', 'p');
$id 			= getUrlParam('HTTP_GET', 'INT', 'id');
$task 			= getUrlParam('HTTP_GET', 'STRING', 'task');
$option 		= getUrlParam('HTTP_GET', 'STRING', 'option');
$date_from	 	= getUrlParam('HTTP_GET', 'STRING', 'date_from');
$date_to 		= getUrlParam('HTTP_GET', 'STRING', 'date_to');
$f_lat			= getUrlParam('HTTP_GET', 'FLOAT', 'lat');
$f_lon			= getUrlParam('HTTP_GET', 'FLOAT', 'lon');
$f_lat_range 	= getUrlParam('HTTP_GET', 'FLOAT', 'lat_range');
$f_lon_range 	= getUrlParam('HTTP_GET', 'FLOAT', 'lon_range');
$description 	= getUrlParam('HTTP_GET', 'STRING', 'description');

// convert lat/lon from float value to database integer format
if($option == "search") {
	$option = "filter";
	$f_lat *= 1000000; 
	$f_lon *= 1000000; 
	$f_lat_range *= 1000000; 
	$f_lon_range *= 1000000; 
}

// handling of sort preference
$sort = handlePreference($sort, 'trkpt_sort');

// handling of order preference
$order = handlePreference($order, 'trkpt_order');

// handling of page number (for pagination)
$p = handlePreference($p, 'trkpt_page');

// handling of sql result limit
if($limit) {
	// calc new page number caused by changed result limit
	$p = round($p * $_COOKIE['trkpt_limit'] / $limit);
	setcookie('trkpt_page', $p, time() + 3600 * $cfg['pref_cookie_lifetime']);
}
$limit = handlePreference($limit, 'trkpt_limit');
// use default from config file if not set in cookie or URL GET parameter
if(!$limit)
	$limit = $cfg['result_table_limit'];

// connect to database
$database = new Mysql($cfg['db_host'], $cfg['db_name'], 
					$cfg['db_user'], $cfg['db_password']);

if(!$cfg['embedded_mode'] || !$cfg['public_host'] || check_password($cfg['admin_password'])) {
	HTML::heading(_APP_NAME, 2);
	HTML::main_menu();
}

switch ($task) {
	case 'view':
		viewTrace($id, $p, $sort, $order, $limit);
		break;

	case 'search':
		searchTP();
		break;

	default:
		viewTrace(NULL, $p, $sort, $order, $limit);
		break;
}

function viewTrace($id, $page, $sort, $order, $limit) {
	global $DEBUG, $cfg, $database;
	global $option, $date_from, $date_to, $f_lat, $f_lon, $f_lat_range, $f_lon_range;
	HTML::heading(_MENU_TRKPT_VIEW, 3);
	$search_url = "";
	
	switch($sort) {
		case '1':
			$qs = 'gpx_id';
			break;
		case '2':
			$qs = 'altitude';
			break;
		case '4':
			$qs = 'fix';
			break;
		case '5':
			$qs = 'sat';
			break;
		case '6':
			$qs = 'hdop';
			break;
		case '7':
			$qs = 'pdop';
			break;
		case '3':
		default:
			$qs = 'timestamp';
			$sort = 3;
			break;
	}
	
	if(!$order)
		$qo = 'ASC';
	else
		$qo = 'DESC';
	
	// set strict MySQL config, useful for optimizing SQL queries
	// could be removed as soon as bug FS#75 is closed
	if($DEBUG) {
		out('SQL_MAX_JOIN_SIZE=150000;', 'OUT_DEBUG');
		$result = $database->query("SET SQL_MAX_JOIN_SIZE=150000;");
		$result = $database->query("SET SQL_BIG_SELECTS=0;");
	}	
	// workaround for bug FS#75, set SQL_BIG_SELECTS=1 on line 12 if necessary
	if(SQL_BIG_SELECTS) {
		$result = $database->query("SET SQL_BIG_SELECTS=1;");
	}
	
	$query = "SELECT SQL_CALC_FOUND_ROWS `tpt`.* , `gpx`.`timezone`, 
				`gpx`.`name` AS 'gpx_filename', `gpx`.`description` AS `gpx_description` 
				FROM `${cfg['db_table_prefix']}gpx_import` AS `tpt` 
				LEFT JOIN `${cfg['db_table_prefix']}gpx_files` AS `gpx` 
				ON `gpx_id` = `gpx`.`id` ";
	if($id) {
		$query .= "WHERE `gpx_id` = '$id' ";
		$search_url = "&id=$id";
	}
	if($option == "filter") {
		$query .= "WHERE 1 ";
		if(strtotime($date_from) || strtotime($date_to)) {
			if(!strtotime($date_from))
				$from = 0; // 1970-01-01
			else
				$from = strtotime($date_from);
			if(!strtotime($date_to))
				$to = 'UNIX_TIMESTAMP(NOW())';
			else
				$to = strtotime($date_to) + 24*3600; // add one day
			$query .= "AND UNIX_TIMESTAMP(`tpt`.`timestamp`) BETWEEN $from AND $to ";
		}
		if($f_lat)
			$query .= "AND `tpt`.`latitude` BETWEEN '".($f_lat - $f_lat_range)."' 
					AND '".($f_lat + $f_lat_range)."' ";
		if($f_lon)
			$query .= "AND `tpt`.`longitude` BETWEEN '".($f_lon - $f_lon_range)."' 
					AND '".($f_lon + $f_lon_range)."' ";

		$search_url = "&option=filter&date_from=$date_from&date_to=$date_to&lat=$f_lat&lon=$f_lon&lat_range=$f_lat_range&lon_range=$f_lon_range";
	}
	$query .= "ORDER BY `$qs` $qo LIMIT ". ($page*$limit) .", $limit ;";
    $result = $database->query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
    if(mysqli_num_rows($result)) {
		$num_found = $database->query("SELECT FOUND_ROWS();")->fetch_array()[0];
		if($option == "filter" || $id)
			HTML::message_r(_CMN_SEARCH_RESULTS, $num_found);
    	
		HTML::viewPagination($page, ceil($num_found/$limit), 
			"traces.php?task=view&s=$sort&o=$order".$search_url);
		HTML::viewResultLimitSelect("traces.php?task=view&id=$id&p=$page&s=$sort&o=$order".$search_url);
		
		HTML_traces::viewTraceTableHeader("traces.php?task=view&id=$id&p=$page".$search_url, $sort, $order);
		while($row = mysqli_fetch_array($result, MYSQLI_ASSOC)) {
			HTML_traces::viewTraceTableRow($row);
		}
		HTML_traces::viewTraceTableFooter();
		
		HTML::viewPagination($page, ceil($num_found/$limit), 
			"traces.php?task=view&s=$sort&o=$order".$search_url);
		if($option != "filter" && !$id) {
			HTML::message($num_found ._DB_TRKPTS_AVAILABLE);
		}elseif(!$cfg['public_host'] || $cfg['enable_gpx_download'] 
			|| check_password($cfg['admin_password'])) {
			HTML::message("<a href='export.php?trkpt=-1$search_url'>". _TRC_EXPORT_AS_GPX ."</a>");
		}
	}
	else {
		// if cookie has a invalid page number then reset it
		if($page > 0) {
			// redirect to 1st page (might be empty, if there are no trkpts)
			HTML::viewLocationRedirectJS('traces.php?task=view&p=0');
		}
		else
			HTML::message(_TRC_NO_TRKPTS_IN_DB);
	}
}

function searchTP() {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_TRKPT_SEARCH, 3);
    HTML_traces::searchTrkPtForm('');
}


if($cfg['show_exec_time']) {
    $endTime = microtime_float();
    $exectime = round($endTime - $startTime, 4);
}
include("./foot.html.php");
?>
